<?php

	require_once(_PS_ROOT_DIR_ . '/idealcheckout/includes/library.php');

	class Idealcheckoutklarnainvoice extends PaymentModule
	{
		public function __construct()
		{
			$this->name = 'idealcheckoutklarnainvoice';
			$this->author = 'iDEAL Checkout';

			if(version_compare(_PS_VERSION_, '1.4', '<'))
			{
				$this->tab = 'Payment';
			}
			else
			{
				$this->tab = 'payments_gateways';
			}

			$this->version = 0.5;
			
			// Only EURO's are allowed!
			$this->currencies = false;
			$this->currencies_mode = 'radio';

			parent::__construct();

			$this->displayName = $this->l('KlarnaInvoice');
			$this->description = $this->l('Online betalen met KlarnaInvoice.');
		}

		// Gateway install tool in admin
		public function install()
		{
			if(!parent::install() || !$this->registerHook('payment') || !$this->registerHook('paymentReturn'))
			{
				return false;
			}

			return true;
		}

		// Gateway uninstall tool in admin
		public function uninstall()
		{
			if(!parent::uninstall() || !$this->registerHook('payment') || !$this->registerHook('paymentReturn'))
			{
				return false;
			}
			
			return true;
		}
	
		// Gateway configuration tool in admin
		public function getContent()
		{
			$aGatewaySettings = idealcheckout_getGatewaySettings(false, 'klarnainvoice');

			$sHtml = '
<p>Via deze plugin kunt u KlarnaInvoice betalingen ontvangen in uw webshop via diverse Payment Service Providers.</p>
<p>Deze plugin is momenteel geconfigureerd om KlarnaInvoice transacties te verwerken via <b>' . htmlspecialchars($aGatewaySettings['GATEWAY_NAME']) . '</b>.<br>Meer informatie over deze Payment Service Provider vind u op <a href="' . htmlspecialchars($aGatewaySettings['GATEWAY_WEBSITE']) . '" target="_blank">' . htmlspecialchars($aGatewaySettings['GATEWAY_WEBSITE']) . '</a></p>';

			if($aGatewaySettings['GATEWAY_VALIDATION'])
			{
				$sHtml .= '
<p>&nbsp;</p>
<p><b>Transacties Controleren</b></p>
<p>Controleer de status van alle openstaande transacties bij uw Payment Service Provider.</p>
<p><input type="button" value="Controleer openstaande transacties." onclick="javascript: window.open(\'' . idealcheckout_getRootUrl(1) . 'idealcheckout/validate.php?gateway=klarnainvoice\', \'popup\', \'directories=no,height=550,location=no,menubar=no,resizable=yes,scrollbars=yes,status=no,toolbar=no,width=750\');"></p>';
			}

			$sHtml .= '
<p>&nbsp;</p>
<p><b>Over deze plugin</b></p>
<p>Deze plugin is ontwikkeld door <a href="https://www.ideal-checkout.nl" target="_blank">iDEAL Checkout</a> en is GRATIS te downloaden via <a href="http://www.ideal-checkout.nl" target="_blank">http://www.ideal-checkout.nl</a>.<br><br>- Voor vragen en support, kijk op <a href="http://www.ideal-checkout.nl" target="_blank">http://www.ideal-checkout.nl</a> of neem contact op met <a href="mailto:info@ideal-checkout.nl">info@ideal-checkout.nl</a>.<br>- Feedback en donaties worden zeer op prijs gesteld.<br>- Het gebruik van onze plugins/scripts is geheel op eigen risico.</p>';

			return $sHtml;
		}



		public function execPayment($cart)
		{
			// Save order in backend
			$iOrderState = 1000; // Awaiting payment
			$this->validateOrder($cart->id, $iOrderState, $cart->getOrderTotal(true, 3), $this->displayName, 'Betaling wordt gestart', array(), false, false, $cart->secure_key);
			$iOrderId = $this->currentOrder;
			$oOrder = new Order($iOrderId);


			$oCustomer = new Customer(intval($cart->id_customer));
			$oPaymentAddress = new Address(intval($cart->id_address_invoice));
			$oShipmentAddress = new Address(intval($cart->id_address_delivery));
			$oCurrency = new Currency(intval($cart->id_currency));

			if(!Validate::isLoadedObject($oPaymentAddress))
			{
				idealcheckout_die('Invalid invoice address.', __FILE__, __LINE__);
			}
			elseif(!Validate::isLoadedObject($oShipmentAddress))
			{
				idealcheckout_die('Invalid shipment address.', __FILE__, __LINE__);
			}
			elseif(!Validate::isLoadedObject($oCustomer))
			{
				idealcheckout_die('Invalid customer.', __FILE__, __LINE__);
			}

			$oPaymentCountry = new Country(intval($oPaymentAddress->id_country));
			$oShipmentCountry = new Country(intval($oShipmentAddress->id_country));

			if(!Validate::isLoadedObject($oPaymentCountry))
			{
				idealcheckout_die('Invalid payment country.', __FILE__, __LINE__);
			}
			elseif(!Validate::isLoadedObject($oShipmentCountry))
			{
				idealcheckout_die('Invalid shipment country.', __FILE__, __LINE__);
			}




			// Load database settings
			$aDatabaseSettings = idealcheckout_getDatabaseSettings();



			$sLanguageCode = '';

			if(is_callable('Context::getContext'))
			{
				$oContect = Context::getContext();

				if(isset($oContect->language->iso_code))
				{
					$sLanguageCode = substr($oContect->language->iso_code, 0, 2);
				}
			}

			if(!in_array($sLanguageCode, array('nl', 'de', 'en')))
			{
				$sLanguageCode = 'nl';
			}


			$sStoreCode = idealcheckout_getStoreCode();
			$sGatewayCode = 'klarnainvoice';
			// $sLanguageCode = ''; // nl, de, en
			$sCountryCode = '';
			$sCurrencyCode = 'EUR';

			$sOrderId = $iOrderId; // $cart->id;
			$sOrderCode = idealcheckout_getRandomCode(32);
			$aOrderParams = array();
			$sTransactionId = idealcheckout_getRandomCode(32);
			$sTransactionCode = idealcheckout_getRandomCode(32);
			$fTransactionAmount = number_format($cart->getOrderTotal(true, 3), 2, '.', '');

			$sRootUrl = _PS_BASE_URL_SSL_ . __PS_BASE_URI__;

			$sTransactionDescription = idealcheckout_getTranslation($sLanguageCode, 'idealcheckout', 'Webshop order #{0}', array($sOrderId));
			$sTransactionPaymentUrl = ''; // $sRootUrl . 'modules/idealcheckoutklarnainvoice/validation.php?order_id=' . $sOrderId . '&order_code=' . $sOrderCode;
			$sTransactionSuccessUrl = $sRootUrl . 'modules/idealcheckoutklarnainvoice/validation.php?order_id=' . $sOrderId . '&order_code=' . $sOrderCode;
			$sTransactionPendingUrl = $sRootUrl . 'modules/idealcheckoutklarnainvoice/validation.php?order_id=' . $sOrderId . '&order_code=' . $sOrderCode;
			$sTransactionFailureUrl = $sRootUrl . 'modules/idealcheckoutklarnainvoice/validation.php?order_id=' . $sOrderId . '&order_code=' . $sOrderCode;


			// Store ORDER information
			$aOrderParams['cart'] = array(
				'id' => $cart->id, 
				'secure_key' => $cart->secure_key
			);




			$aOrderParams['customer'] = array();

			// Shipment data
			list($sStreetName, $sStreetNumber, $sExtension) = idealcheckout_mb_splitAddress($oShipmentAddress->address1);

			$aOrderParams['customer']['shipment_company'] = ''; // $oShipmentAddress->company;
			$aOrderParams['customer']['shipment_name'] = $oShipmentAddress->firstname . ' ' . $oShipmentAddress->lastname;
			$aOrderParams['customer']['shipment_first_name'] = $oShipmentAddress->firstname;
			$aOrderParams['customer']['shipment_last_name'] = $oShipmentAddress->lastname;
			$aOrderParams['customer']['shipment_gender'] = '';
			$aOrderParams['customer']['shipment_date_of_birth'] = '';
			$aOrderParams['customer']['shipment_phone'] = ($oShipmentAddress->phone_mobile ? $oShipmentAddress->phone_mobile : $oShipmentAddress->phone);
			$aOrderParams['customer']['shipment_email'] = $oCustomer->email;
			$aOrderParams['customer']['shipment_address'] = $oShipmentAddress->address1;
			$aOrderParams['customer']['shipment_street_name'] = $sStreetName;
			$aOrderParams['customer']['shipment_street_number'] = $sStreetNumber;
			$aOrderParams['customer']['shipment_street_number_extension'] = $sExtension;
			$aOrderParams['customer']['shipment_zipcode'] = $oShipmentAddress->postcode;
			$aOrderParams['customer']['shipment_city'] = $oShipmentAddress->city;
			$aOrderParams['customer']['shipment_country_code'] = $oShipmentCountry->iso_code;
			$aOrderParams['customer']['shipment_country_name'] = ((strcasecmp($oShipmentCountry->iso_code, 'BE') === 0) ? 'Belgie' : 'Nederland');


			// Payment data
			list($sStreetName, $sStreetNumber, $sExtension) = idealcheckout_mb_splitAddress($oPaymentAddress->address1);

			$aOrderParams['customer']['payment_company'] = ''; // $oPaymentAddress->company;
			$aOrderParams['customer']['payment_name'] = $oPaymentAddress->firstname . ' ' . $oPaymentAddress->lastname;
			$aOrderParams['customer']['payment_first_name'] = $oPaymentAddress->firstname;
			$aOrderParams['customer']['payment_last_name'] = $oPaymentAddress->lastname;
			$aOrderParams['customer']['payment_gender'] = '';
			$aOrderParams['customer']['payment_date_of_birth'] = '';
			$aOrderParams['customer']['payment_phone'] = ($oPaymentAddress->phone_mobile ? $oPaymentAddress->phone_mobile : $oPaymentAddress->phone);
			$aOrderParams['customer']['payment_email'] = $oCustomer->email;
			$aOrderParams['customer']['payment_address'] = $oPaymentAddress->address1;
			$aOrderParams['customer']['payment_street_name'] = $sStreetName;
			$aOrderParams['customer']['payment_street_number'] = $sStreetNumber;
			$aOrderParams['customer']['payment_street_number_extension'] = $sExtension;
			$aOrderParams['customer']['payment_zipcode'] = $oPaymentAddress->postcode;
			$aOrderParams['customer']['payment_city'] = $oPaymentAddress->city;
			$aOrderParams['customer']['payment_country_code'] = $oPaymentCountry->iso_code;
			$aOrderParams['customer']['payment_country_name'] = ((strcasecmp($oPaymentCountry->iso_code, 'BE') === 0) ? 'Belgie' : 'Nederland');

			// Products
			$aOrderParams['products'] = array();

			$fTotalAmountIncl = 0.00;
			$fTotalAmountExcl = 0.00;

			foreach($cart->getProducts() as $k => $aProduct)
			{
				$fPriceIncl = round($aProduct['price_wt'], 2);
				$fPriceExcl = round($aProduct['price'], 2);
				$fPriceVat = round($aProduct['price_wt'] - $aProduct['price'], 2);
				$fVatPercentage = round($aProduct['rate'], 2);

				$fQuantity = $aProduct['cart_quantity'];

				$fTotalAmountIncl += ($fPriceIncl * $fQuantity);
				$fTotalAmountExcl += ($fPriceExcl * $fQuantity);

				$aOrderParams['products'][] = array('code' => $aProduct['id_product'], 'description' => $aProduct['name'], 'quantity' => $fQuantity, 'price_incl' => $fPriceIncl, 'price_excl' => $fPriceExcl, 'vat' => $fVatPercentage);
			}

			// Shipment costs?
			$fShipmentIncl = $cart->getTotalShippingCost();
			$fShipmentExcl = $cart->getTotalShippingCost(NULL, false);

			if(!empty($fShipmentIncl))
			{
				$fShipmentTaxPercentage = round((($fShipmentIncl - $fShipmentExcl) / $fShipmentExcl) * 100, 0);

				$fTotalAmountIncl += $fShipmentIncl;
				$fTotalAmountExcl += $fShipmentExcl;

				$aOrderParams['products'][] = array('code' => 'SHIPPING', 'description' => 'Verzendkosten', 'quantity' => '1', 'price_incl' => $fShipmentIncl, 'price_excl' => $fShipmentExcl, 'vat' => $fShipmentTaxPercentage);
			}


			// Wrapping costs?
			$fWrappingIncl = $cart->getOrderTotal(true, 6);
			$fWrappingExcl = $cart->getOrderTotal(false, 6);

			if(!empty($fWrappingIncl))
			{
				$fWrappingTaxPercentage = round((($fWrappingIncl - $fWrappingExcl) / $fWrappingExcl) * 100, 0);

				$fTotalAmountIncl += $fWrappingIncl;
				$fTotalAmountExcl += $fWrappingExcl;

				$aOrderParams['products'][] = array('code' => 'WRAPPING', 'description' => 'Inpakken', 'quantity' => '1', 'price_incl' => $fWrappingIncl, 'price_excl' => $fWrappingExcl, 'vat' => $fWrappingTaxPercentage);
			}


			// Discounts?
		    $aDiscount = $cart->getDiscounts();
			if(is_array($aDiscount) && !empty($aDiscount))
			{
				$fPriceIncl = 0 - round($aDiscount[0]['value_real'], 2);
				$fPriceExcl = 0 - round($aDiscount[0]['value_tax_exc'], 2);
				$fTaxPercentage = round((($fPriceIncl - $fPriceExcl) / $fPriceExcl) * 100, 0);

				$fTotalAmountIncl += $fPriceIncl;
				$fTotalAmountExcl += $fPriceExcl;

				$aOrderParams['products'][] = array('code' => 'DISCOUNT', 'description' => $aDiscount[0]['description'], 'quantity' => '1', 'price_incl' => $fPriceIncl, 'price_excl' => $fPriceExcl, 'vat' => $fTaxPercentage);
			}

			$fCorrection = round($fTransactionAmount - $fTotalAmountIncl, 2);

			if($fCorrection != 0.00)
			{
				$aOrderParams['products'][] = array('code' => 'CORRECTION', 'description' => 'Correctie', 'quantity' => '1', 'price_incl' => $fCorrection, 'price_excl' => $fCorrection, 'vat' => 0);
			}



			// Insert into #_transactions
			$sql = "INSERT INTO `" . $aDatabaseSettings['table'] . "` SET 
`id` = NULL, 
`order_id` = '" . idealcheckout_escapeSql($sOrderId) . "', 
`order_code` = '" . idealcheckout_escapeSql($sOrderCode) . "', 
`order_params` = '" . idealcheckout_escapeSql(idealcheckout_serialize($aOrderParams)) . "', 
`store_code` = " . (empty($sStoreCode) ? "NULL" : "'" . idealcheckout_escapeSql($sStoreCode) . "'") . ", 
`gateway_code` = '" . idealcheckout_escapeSql($sGatewayCode) . "', 
`language_code` = " . (empty($sLanguageCode) ? "NULL" : "'" . idealcheckout_escapeSql($sLanguageCode) . "'") . ", 
`country_code` = " . (empty($sCountryCode) ? "NULL" : "'" . idealcheckout_escapeSql($sCountryCode) . "'") . ", 
`currency_code` = '" . idealcheckout_escapeSql($sCurrencyCode) . "', 
`transaction_id` = '" . idealcheckout_escapeSql($sTransactionId) . "', 
`transaction_code` = '" . idealcheckout_escapeSql($sTransactionCode) . "', 
`transaction_params` = NULL, 
`transaction_date` = '" . idealcheckout_escapeSql(time()) . "', 
`transaction_amount` = '" . idealcheckout_escapeSql($fTransactionAmount) . "', 
`transaction_description` = '" . idealcheckout_escapeSql($sTransactionDescription) . "', 
`transaction_status` = NULL, 
`transaction_url` = NULL, 
`transaction_payment_url` = " . ($sTransactionPaymentUrl ? "'" . idealcheckout_escapeSql($sTransactionPaymentUrl) . "'" : "NULL") . ", 
`transaction_success_url` = " . ($sTransactionSuccessUrl ? "'" . idealcheckout_escapeSql($sTransactionSuccessUrl) . "'" : "NULL") . ", 
`transaction_pending_url` = " . ($sTransactionPendingUrl ? "'" . idealcheckout_escapeSql($sTransactionPendingUrl) . "'" : "NULL") . ", 
`transaction_failure_url` = " . ($sTransactionFailureUrl ? "'" . idealcheckout_escapeSql($sTransactionFailureUrl) . "'" : "NULL") . ", 
`transaction_log` = NULL;";

			idealcheckout_database_query($sql);


			// Redirect to issuer_request.php
			$sRedirectUrl = $sRootUrl . 'idealcheckout/setup.php?order_id=' . $sOrderId . '&order_code=' . $sOrderCode;



			if(!headers_sent())
			{
				Tools::redirectLink($sRedirectUrl);
				exit;
			}

			return '<script type="text/javascript">document.location.href = \'' . $sRedirectUrl . '\';</script>';
		}




		// Add payment method to payment-page
		public function hookPayment($params)
		{
			if(!$this->active)
			{
				return;
			}

			$sRootUrl = _PS_BASE_URL_SSL_ . __PS_BASE_URI__;

			return '
<p class="payment_module"><a href="' . $sRootUrl . 'modules/idealcheckoutklarnainvoice/payment.php" title="' . htmlentities($this->description) . '"><img alt="' . htmlentities($this->description) . '" border="0" src="' . _PS_BASE_URL_SSL_ . __PS_BASE_URI__ . 'modules/idealcheckoutklarnainvoice/idealcheckoutklarnainvoice.gif" height="49" /> ' . htmlentities($this->description) . '</a></p>';
		}
		
		// Show payment status after payment
		public function hookPaymentReturn($params)
		{
			if(!$this->active)
			{
				return;
			}

			return $this->display(__FILE__, 'order_confirmation.tpl');
		}
	}

?>